@@ -4,15 +4,23 @@
 # Copyright (c) 2017-2021 OWS Contributors
 # SPDX-License-Identifier: Apache-2.0
 from copy import deepcopy
+from typing import Any, MutableMapping
 
 
-def deepinherit(parent, child):
-    expanded = deepcopy(parent)
+def deepinherit(parent: MutableMapping[str, Any], child: MutableMapping[str, Any]) -> MutableMapping[str, Any]:
+    """
+    Implements inheritance for configuration.
+
+    :param parent: The parent configuration to inherit from
+    :param child: The child configuration to override the parent config
+    :return: A new dictionary reflecting the inherited configuration
+    """
+    expanded: MutableMapping[str, Any] = deepcopy(parent)
     deepupdate(expanded, child)
     return expanded
 
 
-def deepupdate(target, src):
+def deepupdate(target: MutableMapping[str, Any], src: MutableMapping[str, Any]) -> None:
     for k in src:
         if isinstance(src[k], dict):
             if k not in target:
