@@ -19,6 +19,7 @@ import os
 from collections.abc import Mapping
 from importlib import import_module
 
+import numpy
 from babel.messages.catalog import Catalog
 from babel.messages.pofile import read_po
 from datacube.utils import geometry
@@ -110,10 +111,12 @@ class BandIndex(OWSMetadataConfig):
             self.add_aliases(self.band_cfg)
         # pylint: disable=attribute-defined-outside-init
         self._nodata_vals = {}
+        self._dtypes = {}
         for b, aliases in self.band_cfg.items():
             if b not in self.native_bands.index:
                 raise ConfigException(f"Unknown band: {b} in layer {self.product.name}")
             self._nodata_vals[b] = self.native_bands['nodata'][b]
+            self._dtypes[b] = numpy.dtype(self.native_bands.dtype[b])
             if isinstance(self._nodata_vals[b], str) and self._nodata_vals[b].lower() == "nan":
                 self._nodata_vals[b] = float("nan")
         super().make_ready(dc, *args, **kwargs)
@@ -141,6 +144,13 @@ class BandIndex(OWSMetadataConfig):
         name = self.band(name_alias)
         return self._nodata_vals[name]
 
+    def dtype_val(self, name_alias):
+        name = self.band(name_alias)
+        return self._dtypes[name]
+
+    def dtype_size(self, name_alias):
+        return self.dtype_val(name_alias).itemsize
+
     def band_labels(self):
         return [self.band_label(b) for b in self.band_cfg]
 
@@ -408,7 +418,7 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
 
         if self.global_cfg.wcs:
             try:
-                self.parse_wcs(cfg.get("wcs"))
+                self.parse_wcs(cfg.get("wcs", {}))
             except KeyError as e:
                 raise ConfigException(f"Missing required config item {e} in wcs section for layer {self.name}")
 
@@ -546,7 +556,7 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
 
     # pylint: disable=attribute-defined-outside-init
     def parse_wcs(self, cfg):
-        if cfg is None or not self.global_cfg.wcs:
+        if not self.global_cfg.wcs:
             self.wcs = False
             return
         else:
@@ -584,8 +594,12 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         self.declare_unready("grid_high_y")
         self.declare_unready("grids")
         # Band management
-        self.wcs_raw_default_bands = cfg["default_bands"]
-        self.declare_unready("wcs_default_bands")
+        if cfg.get("default_bands"):
+            _LOG.warning(
+                "wcs section contains a 'default_bands' list.  WCS default_bands list is no longer supported. "
+                "Functionally, the default behaviour is now to return all available bands (as mandated by "
+                "the WCS2.x spec). "
+            )
 
         # Native format
         if "native_format" in cfg:
@@ -723,8 +737,6 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
                             (bbox["top"] - bbox["bottom"]) / self.grid_high_y
                         )
                     }
-            # Band management
-            self.wcs_default_bands = [self.band_idx.band(b) for b in self.wcs_raw_default_bands]
 
     def parse_product_names(self, cfg):
         raise NotImplementedError()
