@@ -3,8 +3,10 @@
 #
 # Copyright (c) 2017-2021 OWS Contributors
 # SPDX-License-Identifier: Apache-2.0
+import io
 import logging
 from datetime import datetime
+from typing import List, MutableMapping, Optional, Union, cast
 
 import numpy
 from colour import Color
@@ -13,60 +15,81 @@ from matplotlib import patches as mpatches
 from matplotlib import pyplot as plt
 from xarray import DataArray, Dataset, merge
 
-from datacube_ows.config_utils import ConfigException, OWSConfigEntry
+from datacube_ows.config_utils import CFG_DICT, ConfigException, OWSConfigEntry
 from datacube_ows.styles.base import StyleDefBase
 
 _LOG = logging.getLogger(__name__)
 
 
 class ValueMapRule(OWSConfigEntry):
-    def __init__(self, style_def, band, cfg):
-        super().__init__(self, cfg)
+    """
+    A Value Map Rule.
+
+    Construct a ValueMap rule-set with ValueMapRule.value_map_from_config
+    """
+    def __init__(self, style_def: "ColorMapStyleDef", band: str, cfg: CFG_DICT) -> None:
+        """
+        Construct a Value Map Rule
+
+        :param style_def: The owning ColorMapStyleDef object
+        :param band: The name of the flag-band the rules apply to
+        :param cfg: The rule specification
+        """
+        super().__init__(cfg)
+        cfg = cast(CFG_DICT, self._raw_cfg)
         self.style = style_def
         self.band = style_def.local_band(band)
 
-        self.title = cfg["title"]
-        self.abstract = cfg.get("abstract")
+        self.title = cast(str, cfg["title"])
+        self.abstract = cast(str, cfg.get("abstract"))
         if self.title and self.abstract:
-            self.label = f"{self.title} - {self.abstract}"
+            self.label: Optional[str] = f"{self.title} - {self.abstract}"
         elif self.title:
             self.label = self.title
         elif self.abstract:
             self.label = self.abstract
         else:
             self.label = None
-        self.color_str = cfg["color"]
+        self.color_str = cast(str, cfg["color"])
         self.rgb = Color(self.color_str)
         if cfg.get("mask", False):
-            self.alpha = 0.0
+            self.alpha: float = 0.0
         else:
-            self.alpha = cfg.get("alpha", 1.0)
+            self.alpha = float(cast(Union[float, int, str], cfg.get("alpha", 1.0)))
 
         if "flags" in cfg:
-            flags = cfg["flags"]
-            self.or_flags = False
-            if "or" in flags:
+            flags = cast(CFG_DICT, cfg["flags"])
+            self.or_flags: bool = False
+            if "or" in flags and "and" in flags:
+                raise ConfigException(f"ValueMap rule in style {self.style.name} of layer {self.style.product.name} combines 'and' and 'or' rules")
+            elif "or" in flags:
                 self.or_flags = True
-                flags = flags["or"]
+                flags = cast(CFG_DICT, flags["or"])
             elif "and" in flags:
-                flags = flags["and"]
-            self.flags = flags
+                flags = cast(CFG_DICT, flags["and"])
+            self.flags: Optional[CFG_DICT] = flags
         else:
             self.flags = None
             self.or_flags = False
         if "values" in cfg:
-            self.values = cfg["values"]
+            self.values: Optional[List[int]] = cast(List[int], cfg["values"])
         else:
             self.values = None
         if not self.flags and not self.values:
-            raise ConfigException(f"Value map rule in style {style_def.name} of layer {style_def.product.name} must have a non-empty 'flags' or a 'values' section.")
+            raise ConfigException(f"Value map rule in style {style_def.name} of layer {style_def.product.name} must have a non-empty 'flags' or 'values' section.")
         if self.flags and self.values:
-            raise ConfigException(f"Value map rule in style {style_def.name} of layer {style_def.product.name} has a both a 'flags' and a 'values' section - choose one.")
+            raise ConfigException(f"Value map rule in style {style_def.name} of layer {style_def.product.name} has both a 'flags' and a 'values' section - choose one.")
 
-    def create_mask(self, data):
+    def create_mask(self, data: DataArray) -> DataArray:
+        """
+        Create a mask from raw flag band data.
+
+        :param data: Raw flag data, assumed to be for this rule's flag band.
+        :return: A boolean DataArray, True where the data matches this rule
+        """
         if self.values:
-            mask = None
-            for v in self.values:
+            mask: Optional[DataArray] = None
+            for v in cast(List[int], self.values):
                 vmask = data == v
                 if mask is None:
                     mask = vmask
@@ -74,37 +97,63 @@ class ValueMapRule(OWSConfigEntry):
                     mask |= vmask
         elif self.or_flags:
             mask = None
-            for f in self.flags.items():
+            for f in cast(CFG_DICT, self.flags).items():
                 f = {f[0]: f[1]}
                 if mask is None:
                     mask = make_mask(data, **f)
                 else:
                     mask |= make_mask(data, **f)
         else:
-            mask = make_mask(data, **self.flags)
+            mask = make_mask(data, **cast(CFG_DICT, self.flags))
         return mask
 
     @classmethod
-    def value_map_from_config(cls, style, cfg):
-        vmap = {}
+    def value_map_from_config(cls,
+                              style: "ColorMapStyleDef",
+                              cfg: CFG_DICT) -> MutableMapping[str, List["ValueMapRule"]]:
+        """
+        Create a value map rule set from a config specification
+
+        :param style: The parent style definition object
+        :param cfg: The specification for the value map.
+
+        :return: A value map ruleset dictionary.
+        """
+        vmap: MutableMapping[str, List["ValueMapRule"]] = {}
         for band_name, rules in cfg.items():
-            band_rules = [cls(style, band_name, rule) for rule in rules]
+            band_rules = [cls(style, band_name, rule) for rule in cast(List[CFG_DICT], rules)]
             vmap[band_name] = band_rules
         return vmap
 
 
 class ColorMapStyleDef(StyleDefBase):
+    """
+    Style subclass for value-map styles
+    """
     auto_legend = True
 
-    def __init__(self, product, style_cfg, stand_alone=False, user_defined=False):
+    def __init__(self,
+                 product: "datacube_ows.ows_configuration.OWSNamedLayer",
+                 style_cfg: CFG_DICT,
+                 stand_alone: bool = False,
+                 user_defined: bool = False) -> None:
+        """"
+        Constructor - refer to StyleDefBase
+        """
         super().__init__(product, style_cfg, stand_alone=stand_alone, user_defined=user_defined)
-        style_cfg = self._raw_cfg
-        self.value_map = ValueMapRule.value_map_from_config(self, style_cfg["value_map"])
+        style_cfg = cast(CFG_DICT, self._raw_cfg)
+        self.value_map = ValueMapRule.value_map_from_config(self, cast(CFG_DICT, style_cfg["value_map"]))
         for band in self.value_map.keys():
             self.raw_needed_bands.add(band)
 
     @staticmethod
-    def reint(data):
+    def reint(data: DataArray) -> DataArray:
+        """
+        Convert a data-array to int.
+
+        :param data: input data (potentially non-integer)
+        :return: same data cast to integer
+        """
         inted = data.astype("int")
         if hasattr(data, "attrs"):
             attrs = data.attrs
@@ -112,17 +161,25 @@ class ColorMapStyleDef(StyleDefBase):
         return inted
 
     @staticmethod
-    def create_colordata(data, rgb, alpha, mask):
+    def create_colordata(data: DataArray, rgb: Color, alpha: float, mask: DataArray) -> Dataset:
+        """Colour a mask with a given colour/alpha"""
         target = Dataset(coords=data.coords)
         colors = ["red", "green", "blue", "alpha"]
         for color in colors:
             val = alpha if color == "alpha" else getattr(rgb, color)
             c = numpy.full(data.shape, val)
             target[color] = DataArray(c, dims=data.dims, coords=data.coords)
+        # pyre-ignore[6]
         masked = target.where(mask).where(numpy.isfinite(data))  # remask
         return masked
 
-    def transform_single_date_data(self, data):
+    def transform_single_date_data(self, data: Dataset) -> Dataset:
+        """
+        Apply style to raw data to make an RGBA image xarray (single time slice only)
+
+        :param data: Raw data, all bands.
+        :return: RGBA uint8 xarray
+        """
         # pylint: disable=too-many-locals, too-many-branches
         # extent mask data per band to preseve nodata
         _LOG.debug("transform begin %s", datetime.now())
@@ -137,8 +194,8 @@ class ColorMapStyleDef(StyleDefBase):
         for cfg_band, rules in self.value_map.items():
             # Run through each item
             band = self.product.band_idx.band(cfg_band)
+            bdata = cast(DataArray, data[band])
             band_data = Dataset()
-            bdata = data[band]
             if bdata.dtype.kind == 'f':
                 # Convert back to int for bitmasking
                 bdata = ColorMapStyleDef.reint(bdata)
@@ -153,7 +210,12 @@ class ColorMapStyleDef(StyleDefBase):
         imgdata *= 255
         return imgdata.astype('uint8')
 
-    def single_date_legend(self, bytesio):
+    def single_date_legend(self, bytesio: io.BytesIO) -> None:
+        """
+        Write a legend into a bytes buffer as a PNG image.
+
+        :param bytesio:  io.BytesIO byte buffer.
+        """
         patches = []
         for band in self.value_map.keys():
             for rule in reversed(self.value_map[band]):
@@ -176,4 +238,5 @@ class ColorMapStyleDef(StyleDefBase):
         plt.savefig(bytesio, format='png')
 
 
+# Register ColorMapStyleDef as a style subclass.
 StyleDefBase.register_subclass(ColorMapStyleDef, "value_map")
