@@ -16,6 +16,7 @@ import numpy
 from colour import Color
 from matplotlib import pyplot as plt
 from matplotlib.colors import LinearSegmentedColormap, to_hex
+from numpy.typing import NDArray
 from xarray import Dataset
 
 from datacube_ows.config_utils import CFG_DICT
@@ -225,9 +226,8 @@ def colour_ramp_legend(bytesio: io.BytesIO,
         cmap=custom_map,
         orientation="horizontal")
 
-    if ticks:
-        color_bar.set_ticks(list(ticks.keys()))
-        color_bar.set_ticklabels(list(ticks.values()))
+    color_bar.set_ticks(list(ticks.keys()))
+    color_bar.set_ticklabels(list(ticks.values()))
 
     title = colour_ramp.legend_title if colour_ramp.legend_title else default_title
     if colour_ramp.legend_units:
@@ -401,6 +401,7 @@ class ColorRamp:
 
     def parse_legend_ticks(self, cfg: CFG_DICT) -> None:
         # Ticks
+        ticks_handled = False
         if "ticks_every" in cfg:
             if "tick_count" in cfg:
                 raise ConfigException("Cannot use tick count and ticks_every in the same legend")
@@ -412,20 +413,25 @@ class ColorRamp:
                 self.ticks.append(tickval)
                 tickval += delta
             self.ticks.append(tickval)
+            ticks_handled = True
         if "ticks" in cfg:
             if "tick_count" in cfg:
                 raise ConfigException("Cannot use tick count and ticks in the same legend")
             self.ticks = [Decimal(t) for t in cast(List[Union[str, int, float]], cfg["ticks"])]
-        if not self.ticks:
+            ticks_handled = True
+        if not ticks_handled:
             count = int(cast(Union[str, int], cfg.get("tick_count", 1)))
             if count < 0:
                 raise ConfigException("tick_count cannot be negative")
-            delta = self.legend_end - self.legend_begin
-            dcount = Decimal(count)
+            elif count == 0:
+                self.ticks.append(self.legend_begin)
+            else:
+                delta = self.legend_end - self.legend_begin
+                dcount = Decimal(count)
 
-            for i in range(0, count + 1):
-                tickval = self.legend_begin + (Decimal(i) / dcount) * delta
-                self.ticks.append(tickval.quantize(self.rounder, rounding=ROUND_HALF_UP))
+                for i in range(0, count + 1):
+                    tickval = self.legend_begin + (Decimal(i) / dcount) * delta
+                    self.ticks.append(tickval.quantize(self.rounder, rounding=ROUND_HALF_UP))
 
     def parse_legend_tick_labels(self, cfg: CFG_DICT) -> None:
         labels = cast(MutableMapping[str, MutableMapping[str, str]], cfg.get("tick_labels", {}))
@@ -452,12 +458,12 @@ class ColorRamp:
         self.legend_strip_location = cast(List[float],
                                           cfg.get("strip_location", [0.05, 0.5, 0.9, 0.15]))
 
-    def get_value(self, data: Union[float, "xarray.DataArray"], band: str) -> numpy.typing.NDArray:
+    def get_value(self, data: Union[float, "xarray.DataArray"], band: str) -> NDArray:
         return numpy.interp(data, self.values, self.components[band])
 
-    def get_8bit_value(self, data: "xarray.DataArray", band: str) -> numpy.typing.NDArray:
-        val: numpy.typing.NDArray = self.get_value(data, band)
-        val = cast(numpy.typing.NDArray, val * 255)
+    def get_8bit_value(self, data: "xarray.DataArray", band: str) -> NDArray:
+        val: NDArray = self.get_value(data, band)
+        val = cast(NDArray, val * 255)
         return val.astype("uint8")
 
     def apply(self, data: "xarray.DataArray") -> "xarray.Dataset":
