@@ -191,9 +191,9 @@ class WCS1GetCoverageRequest():
                     if b not in style.flag_bands:
                         self.bands.add(b)
             else:
-                self.bands = self.product.wcs_default_bands
+                self.bands = self.product.band_idx.band_labels()
         else:
-            self.bands = self.product.wcs_default_bands
+            self.bands = self.product.band_idx.band_labels()
 
         # Argument: EXCEPTIONS (optional - defaults to XML)
         if "exceptions" in args and args["exceptions"] != "application/vnd.ogc.se_xml":
@@ -292,9 +292,10 @@ class WCS1GetCoverageRequest():
         scale_aff = Affine.scale(xscale, yscale)
         self.affine = trans_aff * scale_aff
         self.geobox = geometry.GeoBox(self.width, self.height, self.affine, self.request_crs)
+        self.ows_stats = bool(args.get("ows_stats"))
 
 
-def get_coverage_data(req):
+def get_coverage_data(req, qprof):
     # pylint: disable=too-many-locals, protected-access
     with cube() as dc:
         if not dc:
@@ -303,9 +304,26 @@ def get_coverage_data(req):
                               req.geobox,
                               req.times,
                               bands=req.bands)
+        qprof.start_event("count-datasets")
         n_datasets = stacker.datasets(dc.index, mode=MVSelectOpts.COUNT)
+        qprof.end_event("count-datasets")
+        qprof["n_datasets"] = n_datasets
+
+        try:
+            req.product.resource_limits.check_wcs(n_datasets,
+                                                  req.geobox.height, req.geobox.width,
+                                                  sum(req.product.band_idx.dtype_size(b) for b in req.bands),
+                                                  len(req.times))
+        except ResourceLimited as e:
+            if e.wcs_hard or req.product.low_res_product_names is None:
+                raise WCS1Exception(
+                    f"This request processes too much data to be served in a reasonable amount of time. ({e}) "
+                    + "Please reduce the bounds of your request and try again.")
+            stacker.resource_limited = True
+            qprof["resource_limited"] = str(e)
         if n_datasets == 0:
             # Return an empty coverage file with full metadata?
+            qprof.start_event("build_empty_dataset")
             cfg = get_config()
             x_range = (req.minx, req.maxx)
             y_range = (req.miny, req.maxy)
@@ -345,22 +363,25 @@ def get_coverage_data(req):
                     yname: yvals,
                 }
             ).astype("int16")
+            qprof.start_event("end_empty_dataset")
+            qprof["write_action"] = "Write Empty"
 
             return n_datasets, data
 
-        try:
-            req.product.resource_limits.check_wcs(n_datasets)
-        except ResourceLimited as e:
-            raise WCS1Exception(
-                f"This request processes too much data to be served in a reasonable amount of time. ({e}) "
-                + "Please reduce the bounds of your request and try again.")
+        qprof.start_event("fetch-datasets")
         datasets = stacker.datasets(index=dc.index)
+        qprof.end_event("fetch-datasets")
+        if qprof.active:
+            qprof["datasets"] = stacker.datasets(dc.index, mode=MVSelectOpts.IDS)
+        qprof.start_event("load-data")
         output = stacker.data(datasets, skip_corrections=True)
+        qprof.end_event("load-data")
 
         # Clean extent flag band from output
         for k, v in output.data_vars.items():
             if k not in req.bands:
                 output = output.drop_vars([k])
+        qprof["write_action"] = "Write Data"
         return n_datasets, output
 
 
